@@ -47,7 +47,6 @@
 #include "brasero-session.h"
 #include "brasero-session-helper.h"
 #include "brasero-burn-options.h"
-#include "brasero-data-options.h"
 #include "brasero-video-options.h"
 #include "brasero-src-image.h"
 #include "brasero-src-selection.h"
@@ -67,9 +66,6 @@ struct _BraseroBurnOptionsPrivate
 {
 	BraseroSessionCfg *session;
 
-	gulong valid_sig;
-	gulong input_sig;
-
 	GtkSizeGroup *size_group;
 
 	GtkWidget *source;
@@ -145,6 +141,14 @@ brasero_burn_options_add_source (BraseroBurnOptions *self,
 	gtk_widget_show (priv->source_placeholder);
 }
 
+/**
+ * brasero_burn_options_add_options:
+ * @dialog: a #BraseroBurnOptions
+ * @options: a #GtkWidget
+ *
+ * Adds some new options to be displayed in the dialog.
+ **/
+
 void
 brasero_burn_options_add_options (BraseroBurnOptions *self,
 				  GtkWidget *options)
@@ -180,17 +184,6 @@ brasero_burn_options_add_burn_button (BraseroBurnOptions *self,
 	return priv->button;
 }
 
-static void
-brasero_burn_options_lock_selection (BraseroBurnOptions *self)
-{
-	BraseroBurnOptionsPrivate *priv;
-
-	priv = BRASERO_BURN_OPTIONS_PRIVATE (self);
-	brasero_medium_selection_set_active (BRASERO_MEDIUM_SELECTION (priv->selection),
-					     brasero_drive_get_medium (brasero_burn_session_get_burner (BRASERO_BURN_SESSION (priv->session))));
-	brasero_dest_selection_lock (BRASERO_DEST_SELECTION (priv->selection), TRUE);
-}
-
 static void
 brasero_burn_options_set_type_shown (BraseroBurnOptions *self,
 				     BraseroMediaType type)
@@ -270,7 +263,6 @@ brasero_burn_options_not_ready_dialog_show_cb (gpointer data)
 
 	priv = BRASERO_BURN_OPTIONS_PRIVATE (data);
 	priv->not_ready_id = 0;
-	gtk_widget_show (priv->status_dialog);
 	return FALSE;
 }
 
@@ -306,6 +298,7 @@ brasero_burn_options_update_valid (BraseroBurnOptions *self)
 					  "response", 
 					  G_CALLBACK (brasero_burn_options_not_ready_dialog_cancel_cb),
 					  self);
+	gtk_widget_show (priv->status_dialog);
 			priv->not_ready_id = g_timeout_add_seconds (1,
 								    brasero_burn_options_not_ready_dialog_show_cb,
 								    self);
@@ -332,7 +325,7 @@ brasero_burn_options_update_valid (BraseroBurnOptions *self)
 
 			message = brasero_notify_message_add (BRASERO_NOTIFY (priv->message_output),
 							      _("Would you like to burn the selection of files across several media?"),
-							      _("The size of the project is too large for the disc even with the overburn option."),
+							      _("The data size is too large for the disc even with the overburn option."),
 							      -1,
 							      BRASERO_NOTIFY_CONTEXT_SIZE);
 			brasero_notify_button_add (BRASERO_NOTIFY (priv->message_output),
@@ -349,7 +342,7 @@ brasero_burn_options_update_valid (BraseroBurnOptions *self)
 		else
 			brasero_notify_message_add (BRASERO_NOTIFY (priv->message_output),
 						    _("Please choose another CD or DVD or insert a new one."),
-						    _("The size of the project is too large for the disc even with the overburn option."),
+						    _("The data size is too large for the disc even with the overburn option."),
 						    -1,
 						    BRASERO_NOTIFY_CONTEXT_SIZE);
 	}
@@ -376,19 +369,19 @@ brasero_burn_options_update_valid (BraseroBurnOptions *self)
 		if (brasero_track_type_get_has_data (type))
 			brasero_notify_message_add (BRASERO_NOTIFY (priv->message_output),
 						    _("Please add files."),
-						    _("The project is empty"),
+						    _("There are no files to write to disc"),
 						    -1,
 						    BRASERO_NOTIFY_CONTEXT_SIZE);
 		else if (!BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (type)))
 			brasero_notify_message_add (BRASERO_NOTIFY (priv->message_output),
 						    _("Please add songs."),
-						    _("The project is empty"),
+						    _("There are no songs to write to disc"),
 						    -1,
 						    BRASERO_NOTIFY_CONTEXT_SIZE);
 		else
 			brasero_notify_message_add (BRASERO_NOTIFY (priv->message_output),
 						     _("Please add videos."),
-						    _("The project is empty"),
+						    _("There are no videos to write to disc"),
 						    -1,
 						    BRASERO_NOTIFY_CONTEXT_SIZE);
 		brasero_track_type_free (type);
@@ -413,8 +406,8 @@ brasero_burn_options_update_valid (BraseroBurnOptions *self)
 		if (priv->message_input) {
 			gtk_widget_show (priv->message_input);
 			message = brasero_notify_message_add (BRASERO_NOTIFY (priv->message_input),
-							      _("Please select an image."),
-							      _("There is no selected image."),
+							      _("Please select a disc image."),
+							      _("There is no selected disc image."),
 							      -1,
 							      BRASERO_NOTIFY_CONTEXT_SIZE);
 		}
@@ -427,7 +420,7 @@ brasero_burn_options_update_valid (BraseroBurnOptions *self)
 			message = brasero_notify_message_add (BRASERO_NOTIFY (priv->message_input),
 							      /* Translators: this is a disc image not a picture */
 							      C_("disc", "Please select another image."),
-							      _("It doesn't appear to be a valid image or a valid cue file."),
+							      _("It doesn't appear to be a valid disc image or a valid cue file."),
 							      -1,
 							      BRASERO_NOTIFY_CONTEXT_SIZE);
 		}
@@ -456,7 +449,7 @@ brasero_burn_options_update_valid (BraseroBurnOptions *self)
 
 		message = brasero_notify_message_add (BRASERO_NOTIFY (priv->message_output),
 						      _("Would you like to burn beyond the disc reported capacity?"),
-						      _("The size of the project is too large for the disc and you must remove files from the project otherwise."
+						      _("The data size is too large for the disc and you must remove files from the selection otherwise."
 							"\nYou may want to use this option if you're using 90 or 100 min CD-R(W) which cannot be properly recognised and therefore need overburn option."
 							"\nNOTE: This option might cause failure."),
 						      -1,
@@ -544,7 +537,7 @@ brasero_burn_options_build_contents (BraseroBurnOptions *object)
 	selection = gtk_hbox_new (FALSE, 12);
 	gtk_widget_show (selection);
 
-	alignment = gtk_alignment_new (0.0, 0.5, 1.0, 0.0);
+	alignment = gtk_alignment_new (0.0, 0.5, 1.0, 1.0);
 	gtk_widget_show (alignment);
 	gtk_box_pack_start (GTK_BOX (selection),
 			    alignment,
@@ -556,7 +549,7 @@ brasero_burn_options_build_contents (BraseroBurnOptions *object)
 	gtk_widget_show (priv->selection);
 	gtk_container_add (GTK_CONTAINER (alignment), priv->selection);
 
-	priv->properties = brasero_medium_properties_new (BRASERO_BURN_SESSION (priv->session));
+	priv->properties = brasero_medium_properties_new (priv->session);
 	gtk_size_group_add_widget (priv->size_group, priv->properties);
 	gtk_widget_show (priv->properties);
 	gtk_box_pack_start (GTK_BOX (selection),
@@ -596,56 +589,14 @@ brasero_burn_options_build_contents (BraseroBurnOptions *object)
 	priv->options = gtk_vbox_new (FALSE, 0);
 	gtk_container_add (GTK_CONTAINER (alignment), priv->options);
 
-	priv->valid_sig = g_signal_connect (priv->session,
-					    "is-valid",
-					    G_CALLBACK (brasero_burn_options_valid_cb),
-					    object);
+	g_signal_connect (priv->session,
+			  "is-valid",
+			  G_CALLBACK (brasero_burn_options_valid_cb),
+			  object);
 
 	brasero_burn_options_update_valid (object);
 }
 
-static void
-brasero_burn_options_finalize (GObject *object)
-{
-	BraseroBurnOptionsPrivate *priv;
-
-	priv = BRASERO_BURN_OPTIONS_PRIVATE (object);
-
-	if (priv->not_ready_id) {
-		g_source_remove (priv->not_ready_id);
-		priv->not_ready_id = 0;
-	}
-
-	if (priv->status_dialog) {
-		gtk_widget_destroy (priv->status_dialog);
-		priv->status_dialog = NULL;
-	}
-
-	if (priv->input_sig) {
-		g_signal_handler_disconnect (priv->session,
-					     priv->input_sig);
-		priv->input_sig = 0;
-	}
-
-	if (priv->valid_sig) {
-		g_signal_handler_disconnect (priv->session,
-					     priv->valid_sig);
-		priv->valid_sig = 0;
-	}
-
-	if (priv->session) {
-		g_object_unref (priv->session);
-		priv->session = NULL;
-	}
-
-	if (priv->size_group) {
-		g_object_unref (priv->size_group);
-		priv->size_group = NULL;
-	}
-
-	G_OBJECT_CLASS (brasero_burn_options_parent_class)->finalize (object);
-}
-
 static void
 brasero_burn_options_reset (BraseroBurnOptions *self)
 {
@@ -740,29 +691,28 @@ brasero_status_dialog_uri_has_image (BraseroTrackDataCfg *track,
 	BraseroBurnOptionsPrivate *priv;
 
 	priv = BRASERO_BURN_OPTIONS_PRIVATE (self);
-
-	name = brasero_utils_get_uri_name (uri);
-	string = g_strdup_printf (_("Do you want to burn \"%s\" to a disc or add it in to the data project?"), name);
 	dialog = gtk_message_dialog_new (GTK_WINDOW (self),
 					 GTK_DIALOG_MODAL |
 					 GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_QUESTION,
 					 GTK_BUTTONS_NONE,
 					 "%s",
-					 string);
+					 _("Do you want to create a disc from the contents of the image or with the image file inside?"));
+	gtk_window_set_title (GTK_WINDOW (dialog), "");
+
+	name = brasero_utils_get_uri_name (uri);
+	/* Translators: %s is the name of the image */
+	string = g_strdup_printf (_("There is only one selected file (\"%s\"). It is the image of a disc and its contents can be burnt."), name);
+	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog), string);
 	g_free (string);
 	g_free (name);
 
-	gtk_window_set_title (GTK_WINDOW (dialog), "");
-	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  _("This file is the image of a disc and can therefore be burnt to disc without having to add it to a data project first."));
+	gtk_dialog_add_button (GTK_DIALOG (dialog), _("Burn as _File"), GTK_RESPONSE_NO);
 
-	gtk_dialog_add_button (GTK_DIALOG (dialog), _("_Add to Project"), GTK_RESPONSE_NO);
-
-	button = brasero_utils_make_button (_("_Burn..."),
-					    NULL,
-					    "media-optical-burn",
-					    GTK_ICON_SIZE_BUTTON);
+	button = brasero_utils_make_button (_("Burn _Contents..."),
+	                                    NULL,
+	                                    "media-optical-burn",
+	                                    GTK_ICON_SIZE_BUTTON);
 	gtk_widget_show (button);
 	gtk_dialog_add_action_widget (GTK_DIALOG (dialog),
 				      button,
@@ -797,7 +747,8 @@ brasero_status_dialog_uri_has_image (BraseroTrackDataCfg *track,
 	track_img = brasero_track_image_cfg_new ();
 	brasero_track_image_cfg_set_source (track_img, uri);
 	brasero_burn_session_add_track (BRASERO_BURN_SESSION (priv->session),
-					BRASERO_TRACK (track_img));
+					BRASERO_TRACK (track_img),
+					NULL);
 
 	return BRASERO_BURN_CANCEL;
 }
@@ -854,7 +805,7 @@ brasero_burn_options_setup_image (BraseroBurnOptions *self)
 	gtk_widget_show (file);
 
 	/* pack everything */
-	string = g_strdup_printf ("<b>%s</b>", _("Select an image to write"));
+	string = g_strdup_printf ("<b>%s</b>", _("Select a disc image to write"));
 	brasero_burn_options_add_source (self, 
 					 string,
 					 file,
@@ -935,14 +886,20 @@ brasero_burn_options_setup (BraseroBurnOptions *self)
 			brasero_burn_options_setup_audio (self);
 	}
 	brasero_track_type_free (type);
-
-	/* see if we should lock the drive only with MERGE */
-	if (brasero_burn_session_get_flags (BRASERO_BURN_SESSION (priv->session)) & BRASERO_BURN_FLAG_MERGE)
-		brasero_burn_options_lock_selection (self);
 }
 
 static void
-brasero_burn_options_input_changed (BraseroBurnSession *session,
+brasero_burn_options_track_added (BraseroBurnSession *session,
+				  BraseroTrack *track,
+				  BraseroBurnOptions *dialog)
+{
+	brasero_burn_options_setup (dialog);
+}
+
+static void
+brasero_burn_options_track_removed (BraseroBurnSession *session,
+				    BraseroTrack *track,
+				    guint former_position,
 				    BraseroBurnOptions *dialog)
 {
 	brasero_burn_options_setup (dialog);
@@ -963,15 +920,18 @@ brasero_burn_options_set_property (GObject *object,
 	switch (prop_id)
 	{
 	case PROP_SESSION: /* Readable and only writable at creation time */
-		priv->session = BRASERO_SESSION_CFG (g_value_get_object (value));
-		g_object_ref (priv->session);
+		priv->session = g_object_ref (g_value_get_object (value));
+
 		g_object_notify (object, "session");
 
-		priv->input_sig = g_signal_connect (priv->session,
-						    "input-changed",
-						    G_CALLBACK (brasero_burn_options_input_changed),
-						    object);
-
+		g_signal_connect (priv->session,
+				  "track-added",
+				  G_CALLBACK (brasero_burn_options_track_added),
+				  object);
+		g_signal_connect (priv->session,
+				  "track-removed",
+				  G_CALLBACK (brasero_burn_options_track_removed),
+				  object);
 		brasero_burn_options_build_contents (BRASERO_BURN_OPTIONS (object));
 		brasero_burn_options_setup (BRASERO_BURN_OPTIONS (object));
 
@@ -1009,6 +969,46 @@ brasero_burn_options_get_property (GObject *object,
 	}
 }
 
+static void
+brasero_burn_options_finalize (GObject *object)
+{
+	BraseroBurnOptionsPrivate *priv;
+
+	priv = BRASERO_BURN_OPTIONS_PRIVATE (object);
+
+	if (priv->not_ready_id) {
+		g_source_remove (priv->not_ready_id);
+		priv->not_ready_id = 0;
+	}
+
+	if (priv->status_dialog) {
+		gtk_widget_destroy (priv->status_dialog);
+		priv->status_dialog = NULL;
+	}
+
+	if (priv->session) {
+		g_signal_handlers_disconnect_by_func (priv->session,
+						      brasero_burn_options_track_added,
+						      object);
+		g_signal_handlers_disconnect_by_func (priv->session,
+						      brasero_burn_options_track_removed,
+						      object);
+		g_signal_handlers_disconnect_by_func (priv->session,
+						      brasero_burn_options_valid_cb,
+						      object);
+
+		g_object_unref (priv->session);
+		priv->session = NULL;
+	}
+
+	if (priv->size_group) {
+		g_object_unref (priv->size_group);
+		priv->size_group = NULL;
+	}
+
+	G_OBJECT_CLASS (brasero_burn_options_parent_class)->finalize (object);
+}
+
 static void
 brasero_burn_options_class_init (BraseroBurnOptionsClass *klass)
 {
@@ -1029,6 +1029,15 @@ brasero_burn_options_class_init (BraseroBurnOptionsClass *klass)
 							      G_PARAM_READWRITE|G_PARAM_CONSTRUCT_ONLY));
 }
 
+/**
+ * brasero_burn_options_new:
+ * @session: a #BraseroSessionCfg object
+ *
+ *  Creates a new #BraseroBurnOptions object.
+ *
+ * Return value: a #GtkWidget object.
+ **/
+
 GtkWidget *
 brasero_burn_options_new (BraseroSessionCfg *session)
 {
