@@ -62,8 +62,6 @@ struct _BraseroDestSelectionPrivate
 	BraseroBurnSession *session;
 
 	BraseroDrive *locked_drive;
-
-	gulong valid_sig;
 };
 
 #define BRASERO_DEST_SELECTION_PRIVATE(o)  (G_TYPE_INSTANCE_GET_PRIVATE ((o), BRASERO_TYPE_DEST_SELECTION, BraseroDestSelectionPrivate))
@@ -75,7 +73,7 @@ enum {
 
 G_DEFINE_TYPE (BraseroDestSelection, brasero_dest_selection, BRASERO_TYPE_MEDIUM_SELECTION);
 
-void
+static void
 brasero_dest_selection_lock (BraseroDestSelection *self,
 			     gboolean locked)
 {
@@ -83,12 +81,16 @@ brasero_dest_selection_lock (BraseroDestSelection *self,
 
 	priv = BRASERO_DEST_SELECTION_PRIVATE (self);
 
-	gtk_widget_set_sensitive (GTK_WIDGET (self), (locked != TRUE));
+	if (locked == (priv->locked_drive != NULL))
+		return;
 
+	gtk_widget_set_sensitive (GTK_WIDGET (self), (locked != TRUE));
 	gtk_widget_queue_draw (GTK_WIDGET (self));
+
 	if (priv->locked_drive) {
 		brasero_drive_unlock (priv->locked_drive);
 		g_object_unref (priv->locked_drive);
+		priv->locked_drive = NULL;
 	}
 
 	if (locked) {
@@ -112,6 +114,14 @@ brasero_dest_selection_lock (BraseroDestSelection *self,
 static void
 brasero_dest_selection_valid_session (BraseroSessionCfg *session,
 				      BraseroDestSelection *self)
+{
+	brasero_medium_selection_update_media_string (BRASERO_MEDIUM_SELECTION (self));
+}
+
+static void
+brasero_dest_selection_output_changed (BraseroSessionCfg *session,
+				       BraseroMedium *former,
+				       BraseroDestSelection *self)
 {
 	BraseroDestSelectionPrivate *priv;
 	BraseroMedium *medium;
@@ -123,12 +133,22 @@ brasero_dest_selection_valid_session (BraseroSessionCfg *session,
 	burner = brasero_burn_session_get_burner (priv->session);
 	medium = brasero_medium_selection_get_active (BRASERO_MEDIUM_SELECTION (self));
 	if (burner != brasero_medium_get_drive (medium))
-		brasero_medium_selection_set_active (BRASERO_MEDIUM_SELECTION (self), medium);
+		brasero_medium_selection_set_active (BRASERO_MEDIUM_SELECTION (self),
+						     brasero_drive_get_medium (burner));
 
 	if (medium)
 		g_object_unref (medium);
+}
 
-	brasero_medium_selection_update_media_string (BRASERO_MEDIUM_SELECTION (self));
+static void
+brasero_dest_selection_flags_changed (BraseroBurnSession *session,
+				      BraseroDestSelection *self)
+{
+	BraseroDestSelectionPrivate *priv;
+
+	priv = BRASERO_DEST_SELECTION_PRIVATE (self);
+
+	brasero_dest_selection_lock (self, (brasero_burn_session_get_flags (BRASERO_BURN_SESSION (priv->session)) & BRASERO_BURN_FLAG_MERGE) != 0);
 }
 
 static void
@@ -179,19 +199,23 @@ brasero_dest_selection_init (BraseroDestSelection *object)
 }
 
 static void
-brasero_dest_selection_finalize (GObject *object)
+brasero_dest_selection_clean (BraseroDestSelection *self)
 {
 	BraseroDestSelectionPrivate *priv;
 
-	priv = BRASERO_DEST_SELECTION_PRIVATE (object);
-
-	if (priv->valid_sig) {
-		g_signal_handler_disconnect (priv->session,
-					     priv->valid_sig);
-		priv->valid_sig = 0;
-	}
+	priv = BRASERO_DEST_SELECTION_PRIVATE (self);
 
 	if (priv->session) {
+		g_signal_handlers_disconnect_by_func (priv->session,
+						      brasero_dest_selection_valid_session,
+						      self);
+		g_signal_handlers_disconnect_by_func (priv->session,
+						      brasero_dest_selection_output_changed,
+						      self);
+		g_signal_handlers_disconnect_by_func (priv->session,
+						      brasero_dest_selection_flags_changed,
+						      self);
+
 		g_object_unref (priv->session);
 		priv->session = NULL;
 	}
@@ -199,13 +223,20 @@ brasero_dest_selection_finalize (GObject *object)
 	if (priv->locked_drive) {
 		brasero_drive_unlock (priv->locked_drive);
 		g_object_unref (priv->locked_drive);
+		priv->locked_drive = NULL;
 	}
+}
 
+static void
+brasero_dest_selection_finalize (GObject *object)
+{
+	brasero_dest_selection_clean (BRASERO_DEST_SELECTION (object));
 	G_OBJECT_CLASS (brasero_dest_selection_parent_class)->finalize (object);
 }
 
 static goffset
-_get_medium_free_space (BraseroMedium *medium)
+_get_medium_free_space (BraseroMedium *medium,
+                        goffset session_blocks)
 {
 	BraseroMedia media;
 	goffset blocks = 0;
@@ -214,15 +245,15 @@ _get_medium_free_space (BraseroMedium *medium)
 	media = brasero_burn_library_get_media_capabilities (media);
 
 	/* NOTE: we always try to blank a medium when we can */
-	if (media & BRASERO_MEDIUM_REWRITABLE)
-		brasero_medium_get_free_space (medium,
-					       NULL,
-					       &blocks);
-	else {
-		brasero_medium_get_free_space (medium,
-					       NULL,
-					       &blocks);
-	}
+	brasero_medium_get_free_space (medium,
+				       NULL,
+				       &blocks);
+
+	if ((media & BRASERO_MEDIUM_REWRITABLE)
+	&& blocks < session_blocks)
+		brasero_medium_get_capacity (medium,
+		                             NULL,
+		                             &blocks);
 
 	return blocks;
 }
@@ -232,8 +263,8 @@ brasero_dest_selection_foreach_medium (BraseroMedium *medium,
 				       gpointer callback_data)
 {
 	BraseroBurnSession *session;
-	goffset session_blocks;
-	goffset burner_blocks;
+	goffset session_blocks = 0;
+	goffset burner_blocks = 0;
 	goffset medium_blocks;
 	BraseroDrive *burner;
 
@@ -287,12 +318,12 @@ brasero_dest_selection_foreach_medium (BraseroMedium *medium,
 choose_closest_size:
 
 	brasero_burn_session_get_size (session, &session_blocks, NULL);
-	medium_blocks = _get_medium_free_space (medium);
+	medium_blocks = _get_medium_free_space (medium, session_blocks);
 
 	if (medium_blocks - session_blocks <= 0)
 		return TRUE;
 
-	burner_blocks = _get_medium_free_space (brasero_drive_get_medium (burner));
+	burner_blocks = _get_medium_free_space (brasero_drive_get_medium (burner), session_blocks);
 	if (burner_blocks - session_blocks <= 0)
 		brasero_burn_session_set_burner (session, brasero_medium_get_drive (medium));
 	else if (burner_blocks - session_blocks > medium_blocks - session_blocks)
@@ -322,6 +353,54 @@ brasero_dest_selection_choose_best (BraseroDestSelection *self)
 	}
 }
 
+void
+brasero_dest_selection_set_session (BraseroDestSelection *selection,
+				    BraseroBurnSession *session)
+{
+	BraseroDestSelectionPrivate *priv;
+
+	priv = BRASERO_DEST_SELECTION_PRIVATE (selection);
+
+	if (priv->session)
+		brasero_dest_selection_clean (selection);
+
+	if (!session)
+		return;
+
+	priv->session = g_object_ref (session);
+	if (brasero_burn_session_get_flags (session) & BRASERO_BURN_FLAG_MERGE) {
+		BraseroDrive *drive;
+
+		drive = brasero_burn_session_get_burner (session);
+		brasero_medium_selection_set_active (BRASERO_MEDIUM_SELECTION (selection),
+						     brasero_drive_get_medium (drive));
+	}
+	else {
+		BraseroMedium *medium;
+
+		medium = brasero_medium_selection_get_active (BRASERO_MEDIUM_SELECTION (selection));
+		if (medium) {
+			brasero_burn_session_set_burner (session, brasero_medium_get_drive (medium));
+			g_object_unref (medium);
+		}
+	}
+
+	g_signal_connect (session,
+			  "is-valid",
+			  G_CALLBACK (brasero_dest_selection_valid_session),
+			  selection);
+	g_signal_connect (session,
+			  "output-changed",
+			  G_CALLBACK (brasero_dest_selection_output_changed),
+			  selection);
+	g_signal_connect (session,
+			  "flags-changed",
+			  G_CALLBACK (brasero_dest_selection_flags_changed),
+			  selection);
+
+	brasero_medium_selection_update_media_string (BRASERO_MEDIUM_SELECTION (selection));
+}
+
 static void
 brasero_dest_selection_set_property (GObject *object,
 				     guint property_id,
@@ -338,32 +417,7 @@ brasero_dest_selection_set_property (GObject *object,
 		/* NOTE: no need to unref a potential previous session since
 		 * it's only set at construct time */
 		session = g_value_get_object (value);
-		priv->session = session;
-		g_object_ref (session);
-
-		if (brasero_burn_session_get_flags (session) & BRASERO_BURN_FLAG_MERGE) {
-			BraseroDrive *drive;
-
-			drive = brasero_burn_session_get_burner (session);
-			brasero_medium_selection_set_active (BRASERO_MEDIUM_SELECTION (object),
-							     brasero_drive_get_medium (drive));
-		}
-		else {
-			BraseroMedium *medium;
-
-			medium = brasero_medium_selection_get_active (BRASERO_MEDIUM_SELECTION (object));
-			if (medium) {
-				brasero_burn_session_set_burner (session, brasero_medium_get_drive (medium));
-				g_object_unref (medium);
-			}
-		}
-
-		priv->valid_sig = g_signal_connect (session,
-						    "is-valid",
-						    G_CALLBACK (brasero_dest_selection_valid_session),
-						    object);
-
-		brasero_medium_selection_update_media_string (BRASERO_MEDIUM_SELECTION (object));
+		brasero_dest_selection_set_session (BRASERO_DEST_SELECTION (object), session);
 		break;
 
 	default:
@@ -406,29 +460,15 @@ brasero_dest_selection_get_output_path (BraseroDestSelection *self)
 	case BRASERO_IMAGE_FORMAT_BIN:
 		brasero_burn_session_get_output (priv->session,
 						 &path,
-						 NULL,
 						 NULL);
 		break;
 
 	case BRASERO_IMAGE_FORMAT_CLONE:
-		brasero_burn_session_get_output (priv->session,
-						 NULL,
-						 &path,
-						 NULL);
-		break;
-
 	case BRASERO_IMAGE_FORMAT_CDRDAO:
-		brasero_burn_session_get_output (priv->session,
-						 NULL,
-						 &path,
-						 NULL);
-		break;
-
 	case BRASERO_IMAGE_FORMAT_CUE:
 		brasero_burn_session_get_output (priv->session,
 						 NULL,
-						 &path,
-						 NULL);
+						 &path);
 		break;
 
 	default:
@@ -516,20 +556,21 @@ brasero_dest_selection_format_medium_string (BraseroMediumSelection *selection,
 				       &data_blocks,
 				       &session_bytes);
 
+	/* Determine the size available for burning */
 	if (flags & (BRASERO_BURN_FLAG_MERGE|BRASERO_BURN_FLAG_APPEND)) {
 		brasero_medium_get_free_space (medium,
 					       &size_bytes,
 					       &blocks);
 	}
 	else {
-		brasero_medium_get_free_space (medium, NULL, &blocks);
+		brasero_medium_get_free_space (medium, &size_bytes, &blocks);
 
 		/* if data would not fit, try to see if once blanked it would
 		 * work. This also covers the case where the media is closed as
 		 * its free space would be 0. This is the best way to do it
 		 * instead of checking for a CLOSED medium as it allows the 
 		 * overwrite media to be appended or merged if need be. */
-		if ((!blocks || blocks > data_blocks)
+		if ((flags & BRASERO_BURN_FLAG_BLANK_BEFORE_WRITE)
 		&& (brasero_burn_library_get_media_capabilities (media) & BRASERO_MEDIUM_REWRITABLE))
 			brasero_medium_get_capacity (medium,
 						     &size_bytes,
@@ -561,6 +602,18 @@ brasero_dest_selection_format_medium_string (BraseroMediumSelection *selection,
 
 	/* format the size */
 	if (brasero_track_type_get_has_stream (input)
+	&& BRASERO_STREAM_FORMAT_HAS_VIDEO (brasero_track_type_get_stream_format (input))) {
+		guint64 free_time;
+
+		/* This is an embarassing problem: this is an approximation
+		 * based on the fact that 2 hours = 4.3GiB */
+		free_time = size_bytes - session_bytes;
+		free_time = free_time * 72000LL / 47LL;
+		size_string = brasero_units_get_time_string (free_time,
+							     TRUE,
+							     TRUE);
+	}
+	else if (brasero_track_type_get_has_stream (input)
 	|| (brasero_track_type_get_has_medium (input)
 	&& (brasero_track_type_get_medium_type (input) & BRASERO_MEDIUM_HAS_AUDIO)))
 		size_string = brasero_units_get_time_string (BRASERO_BYTES_TO_DURATION (size_bytes - session_bytes),
@@ -600,14 +653,12 @@ brasero_dest_selection_class_init (BraseroDestSelectionClass *klass)
 							      "The session",
 							      "The session to work with",
 							      BRASERO_TYPE_BURN_SESSION,
-							      G_PARAM_READWRITE|G_PARAM_CONSTRUCT_ONLY));
+							      G_PARAM_READWRITE));
 }
 
 GtkWidget *
 brasero_dest_selection_new (BraseroBurnSession *session)
 {
-	g_return_val_if_fail (BRASERO_IS_BURN_SESSION (session), NULL);
-
 	return g_object_new (BRASERO_TYPE_DEST_SELECTION,
 			     "session", session,
 			     NULL);
