@@ -89,8 +89,17 @@ G_DEFINE_TYPE (BraseroSessionCfg, brasero_session_cfg, BRASERO_TYPE_SESSION_SPAN
 #define BRASERO_DRIVE_PROPERTIES_KEY		"/apps/brasero/drives"
 
 /**
- * Manages the output path
- */
+ * brasero_session_cfg_has_default_output_path:
+ * @cfg: a #BraseroSessionCfg
+ *
+ * This function returns whether the path returned
+ * by brasero_burn_session_get_output () is an 
+ * automatically created one.
+ *
+ * Return value: a #gboolean. TRUE if the path(s)
+ * creation is handled by @session, FALSE if it was
+ * set.
+ **/
 
 gboolean
 brasero_session_cfg_has_default_output_path (BraseroSessionCfg *session)
@@ -115,7 +124,7 @@ brasero_session_cfg_wrong_extension_signal (BraseroSessionCfg *session)
 	
 	return_value.g_type = 0;
 	g_value_init (&return_value, G_TYPE_BOOLEAN);
-	g_value_set_int (&return_value, FALSE);
+	g_value_set_boolean (&return_value, FALSE);
 
 	g_signal_emitv (instance_and_params,
 			session_cfg_signals [WRONG_EXTENSION_SIGNAL],
@@ -213,8 +222,8 @@ brasero_session_cfg_get_output_path (BraseroBurnSession *session,
 	klass = BRASERO_BURN_SESSION_CLASS (brasero_session_cfg_parent_class);
 
 	result = klass->get_output_path (session,
-					 toc,
-					 image);
+					 image,
+					 toc);
 	if (result == BRASERO_BURN_OK)
 		return result;
 
@@ -338,6 +347,16 @@ brasero_session_cfg_get_gconf_key (BraseroSessionCfg *self,
 	return key;
 }
 
+/**
+ * brasero_session_cfg_get_error:
+ * @cfg: a #BraseroSessionCfg
+ *
+ * This function returns the current status and if
+ * autoconfiguration is/was successful.
+ *
+ * Return value: a #BraseroSessionError.
+ **/
+
 BraseroSessionError
 brasero_session_cfg_get_error (BraseroSessionCfg *self)
 {
@@ -352,6 +371,14 @@ brasero_session_cfg_get_error (BraseroSessionCfg *self)
 	return priv->is_valid;
 }
 
+/**
+ * brasero_session_cfg_disable:
+ * @cfg: a #BraseroSessionCfg
+ *
+ * This function disables autoconfiguration
+ *
+ **/
+
 void
 brasero_session_cfg_disable (BraseroSessionCfg *self)
 {
@@ -361,6 +388,23 @@ brasero_session_cfg_disable (BraseroSessionCfg *self)
 	priv->disabled = TRUE;
 }
 
+/**
+ * brasero_session_cfg_enable:
+ * @cfg: a #BraseroSessionCfg
+ *
+ * This function (re)-enables autoconfiguration
+ *
+ **/
+
+void
+brasero_session_cfg_enable (BraseroSessionCfg *self)
+{
+	BraseroSessionCfgPrivate *priv;
+
+	priv = BRASERO_SESSION_CFG_PRIVATE (self);
+	priv->disabled = FALSE;
+}
+
 static void
 brasero_session_cfg_save_drive_flags (BraseroSessionCfg *self,
 				      BraseroMedium *medium)
@@ -1199,6 +1243,8 @@ brasero_session_cfg_flags_changed (BraseroBurnSession *session)
 	BraseroSessionCfgPrivate *priv;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (session);
+	if (priv->disabled)
+		return;
 
 	/* when we update the flags we don't want a
 	 * whole series of "flags-changed" emitted.
@@ -1207,6 +1253,15 @@ brasero_session_cfg_flags_changed (BraseroBurnSession *session)
 		g_signal_stop_emission_by_name (session, "flags-changed");
 }
 
+/**
+ * brasero_session_cfg_add_flags:
+ * @cfg: a #BraseroSessionCfg
+ * @flags: a #BraseroBurnFlag
+ *
+ * Adds all flags from @flags that are not supported.
+ *
+ **/
+
 void
 brasero_session_cfg_add_flags (BraseroSessionCfg *self,
 			       BraseroBurnFlag flags)
@@ -1239,6 +1294,15 @@ brasero_session_cfg_add_flags (BraseroSessionCfg *self,
 				    FALSE);
 }
 
+/**
+ * brasero_session_cfg_remove_flags:
+ * @cfg: a #BraseroSessionCfg
+ * @flags: a #BraseroBurnFlag
+ *
+ * Removes all flags that are not compulsory.
+ *
+ **/
+
 void
 brasero_session_cfg_remove_flags (BraseroSessionCfg *self,
 				  BraseroBurnFlag flags)
@@ -1265,24 +1329,46 @@ brasero_session_cfg_remove_flags (BraseroSessionCfg *self,
 				    FALSE);
 }
 
+/**
+ * brasero_session_cfg_is_supported:
+ * @cfg: a #BraseroSessionCfg
+ * @flag: a #BraseroBurnFlag
+ *
+ * Checks whether a particular flag is supported.
+ *
+ * Return value: a #gboolean. TRUE if it is supported;
+ * FALSE otherwise.
+ **/
+
 gboolean
 brasero_session_cfg_is_supported (BraseroSessionCfg *self,
-				  BraseroBurnFlag flags)
+				  BraseroBurnFlag flag)
 {
 	BraseroSessionCfgPrivate *priv;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
-	return (priv->supported & flags) == flags;
+	return (priv->supported & flag) == flag;
 }
 
+/**
+ * brasero_session_cfg_is_compulsory:
+ * @cfg: a #BraseroSessionCfg
+ * @flag: a #BraseroBurnFlag
+ *
+ * Checks whether a particular flag is compulsory.
+ *
+ * Return value: a #gboolean. TRUE if it is compulsory;
+ * FALSE otherwise.
+ **/
+
 gboolean
 brasero_session_cfg_is_compulsory (BraseroSessionCfg *self,
-				   BraseroBurnFlag flags)
+				   BraseroBurnFlag flag)
 {
 	BraseroSessionCfgPrivate *priv;
 
 	priv = BRASERO_SESSION_CFG_PRIVATE (self);
-	return (priv->compulsory & flags) == flags;
+	return (priv->compulsory & flag) == flag;
 }
 
 static void
@@ -1375,6 +1461,14 @@ brasero_session_cfg_class_init (BraseroSessionCfgClass *klass)
 		              G_TYPE_NONE);
 }
 
+/**
+ * brasero_session_cfg_new:
+ *
+ *  Creates a new #BraseroSessionCfg object.
+ *
+ * Return value: a #BraseroSessionCfg object.
+ **/
+
 BraseroSessionCfg *
 brasero_session_cfg_new (void)
 {
