@@ -55,6 +55,9 @@ struct _BraseroMediumMonitorPrivate
 	GSList *drives;
 	GVolumeMonitor *gmonitor;
 
+	GSList *waiting_removal;
+	guint waiting_removal_id;
+
 	gint probing;
 };
 
@@ -273,14 +276,11 @@ brasero_medium_monitor_get_media (BraseroMediumMonitor *monitor,
 		}
 
 		if (type & BRASERO_MEDIA_TYPE_FILE) {
-			BraseroDrive *drive;
-
 			/* make sure the drive is indeed a fake one
 			 * since it can happen that some medium did
 			 * not properly carry out their initialization 
 			 * and are flagged as BRASERO_MEDIUM_FILE
 			 * whereas they are not */
-			drive = brasero_medium_get_drive (medium);
 			if (brasero_drive_is_fake (drive)) {
 				list = g_slist_prepend (list, medium);
 				g_object_ref (medium);
@@ -315,22 +315,18 @@ brasero_medium_monitor_medium_removed_cb (BraseroDrive *drive,
 
 static gboolean
 brasero_medium_monitor_is_drive (BraseroMediumMonitor *monitor,
-                                 GDrive *gdrive)
+                                 const gchar *device)
 {
 	BraseroMediumMonitorPrivate *priv;
 	BraseroDeviceHandle *handle;
 	BraseroScsiErrCode code;
 	gboolean result;
-	gchar *device;
 
 	priv = BRASERO_MEDIUM_MONITOR_PRIVATE (monitor);
 
-	device = g_drive_get_identifier (gdrive, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
 	BRASERO_MEDIA_LOG ("Testing drive %s", device);
 
 	handle = brasero_device_handle_open (device, FALSE, &code);
-	g_free (device);
-
 	if (!handle)
 		return FALSE;
 
@@ -343,20 +339,41 @@ brasero_medium_monitor_is_drive (BraseroMediumMonitor *monitor,
 }
 
 static void
-brasero_medium_monitor_connected_cb (GVolumeMonitor *monitor,
-                                     GDrive *gdrive,
-                                     BraseroMediumMonitor *self)
+brasero_medium_monitor_device_added (BraseroMediumMonitor *self,
+                                     const gchar *device,
+                                     GDrive *gdrive)
 {
 	BraseroMediumMonitorPrivate *priv;
 	BraseroDrive *drive = NULL;
 
-	if (!brasero_medium_monitor_is_drive (self, gdrive))
+	priv = BRASERO_MEDIUM_MONITOR_PRIVATE (self);
+
+	/* See if the drive is waiting removal.
+	 * This is necessary as GIO behaves strangely sometimes
+	 * since it sends the "disconnected" signal when a medium
+	 * is removed soon followed by a "connected" signal */
+	drive = brasero_medium_monitor_get_drive (self, device);
+	if (drive) {
+		/* Just in case that drive was waiting removal */
+		priv->waiting_removal = g_slist_remove (priv->waiting_removal, drive);
+
+		BRASERO_MEDIA_LOG ("Added signal was emitted but the drive is in the removal list. Updating GDrive associated object.");
+		g_object_set (drive,
+		              "gdrive", gdrive,
+		              NULL);
+
+		g_object_unref (drive);
+		return;
+	}
+
+	/* Make sure it's an optical drive */
+	if (!brasero_medium_monitor_is_drive (self, device))
 		return;
 
 	BRASERO_MEDIA_LOG ("New drive added");
 
-	priv = BRASERO_MEDIUM_MONITOR_PRIVATE (self);
 	drive = g_object_new (BRASERO_TYPE_DRIVE,
+	                      "device", device,
 	                      "gdrive", gdrive,
 	                      NULL);
 	priv->drives = g_slist_prepend (priv->drives, drive);
@@ -385,46 +402,173 @@ brasero_medium_monitor_connected_cb (GVolumeMonitor *monitor,
 			       brasero_drive_get_medium (drive));
 }
 
+static void
+brasero_medium_monitor_connected_cb (GVolumeMonitor *monitor,
+                                     GDrive *gdrive,
+                                     BraseroMediumMonitor *self)
+{
+	gchar *device;
+
+	BRASERO_MEDIA_LOG ("GDrive addition signal");
+
+	device = g_drive_get_identifier (gdrive, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	brasero_medium_monitor_device_added (self, device, gdrive);
+	g_free (device);
+}
+
+static void
+brasero_medium_monitor_volume_added_cb (GVolumeMonitor *monitor,
+                                        GVolume *gvolume,
+                                        BraseroMediumMonitor *self)
+{
+	gchar *device;
+	GDrive *gdrive;
+
+	BRASERO_MEDIA_LOG ("GVolume addition signal");
+
+	/* No need to signal that addition if the GVolume
+	 * object has an associated GDrive as this is just
+	 * meant to trap blank discs which have no GDrive
+	 * associated but a GVolume. */
+	gdrive = g_volume_get_drive (gvolume);
+	if (gdrive) {
+		g_object_unref (gdrive);
+		return;
+	}
+
+	device = g_volume_get_identifier (gvolume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	brasero_medium_monitor_device_added (self, device, NULL);
+	g_free (device);
+}
+
+static gboolean
+brasero_medium_monitor_disconnected_real (gpointer data)
+{
+	BraseroMediumMonitor *self = BRASERO_MEDIUM_MONITOR (data);
+	BraseroMediumMonitorPrivate *priv;
+	BraseroMedium *medium;
+	BraseroDrive *drive;
+
+	priv = BRASERO_MEDIUM_MONITOR_PRIVATE (self);
+
+	if (!priv->waiting_removal) {
+		priv->waiting_removal_id = 0;
+		return FALSE;
+	}
+
+	drive = priv->waiting_removal->data;
+	priv->waiting_removal = g_slist_remove (priv->waiting_removal, drive);
+
+	BRASERO_MEDIA_LOG ("Drive removed");
+	medium = brasero_drive_get_medium (drive);
+
+	/* disconnect the signal handlers to avoid having the "medium-removed" fired twice */
+	g_signal_handlers_disconnect_by_func (drive,
+	                                      brasero_medium_monitor_medium_added_cb,
+	                                      self);
+	g_signal_handlers_disconnect_by_func (drive,
+	                                      brasero_medium_monitor_medium_removed_cb,
+	                                      self);
+
+	if (medium)
+		g_signal_emit (self,
+			       medium_monitor_signals [MEDIUM_REMOVED],
+			       0,
+			       medium);
+
+	priv->drives = g_slist_remove (priv->drives, drive);
+	g_signal_emit (self,
+		       medium_monitor_signals [DRIVE_REMOVED],
+		       0,
+		       drive);
+	g_object_unref (drive);
+
+	/* in case there are more */
+	return TRUE;
+}
+
+static void
+brasero_medium_monitor_device_removed (BraseroMediumMonitor *self,
+                                       const gchar *device,
+                                       GDrive *gdrive)
+{
+	BraseroMediumMonitorPrivate *priv;
+	GDrive *associated_gdrive;
+	BraseroDrive *drive;
+
+	priv = BRASERO_MEDIUM_MONITOR_PRIVATE (self);
+
+	/* Make sure it's one already detected */
+	/* GIO behaves strangely: every time a medium 
+	 * is removed from a drive it emits the disconnected
+	 * signal (which IMO it shouldn't) soon followed by
+	 * a connected signal.
+	 * So delay the removal by one or two seconds. */
+
+	drive = brasero_medium_monitor_get_drive (self, device);
+	if (!drive)
+		return;
+
+	if (G_UNLIKELY (g_slist_find (priv->waiting_removal, drive) != NULL)) {
+		g_object_unref (drive);
+		return;
+	}
+
+	associated_gdrive = brasero_drive_get_gdrive (drive);
+	if (associated_gdrive == gdrive) {
+		BRASERO_MEDIA_LOG ("Found device to remove");
+		priv->waiting_removal = g_slist_append (priv->waiting_removal, drive);
+
+		if (!priv->waiting_removal_id)
+			priv->waiting_removal_id = g_timeout_add_seconds (2,
+			                                                  brasero_medium_monitor_disconnected_real, 
+			                                                  self);
+	}
+	else if (associated_gdrive) {
+		/* do nothing and wait for a "drive-disconnected" signal */
+		g_object_unref (associated_gdrive);
+	}
+
+	g_object_unref (drive);
+}
+
+static void
+brasero_medium_monitor_volume_removed_cb (GVolumeMonitor *monitor,
+                                          GVolume *gvolume,
+                                          BraseroMediumMonitor *self)
+{
+	gchar *device;
+	GDrive *gdrive;
+
+	BRASERO_MEDIA_LOG ("Volume removal signal");
+
+	/* No need to signal that removal if the GVolume
+	 * object has an associated GDrive as this is just
+	 * meant to trap blank discs which have no GDrive
+	 * associated but a GVolume. */
+	gdrive = g_volume_get_drive (gvolume);
+	if (gdrive) {
+		g_object_unref (gdrive);
+		return;
+	}
+
+	device = g_volume_get_identifier (gvolume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	brasero_medium_monitor_device_removed (self, device, NULL);
+	g_free (device);
+}
+
 static void
 brasero_medium_monitor_disconnected_cb (GVolumeMonitor *monitor,
                                         GDrive *gdrive,
                                         BraseroMediumMonitor *self)
 {
-	BraseroMediumMonitorPrivate *priv;
-	GSList *next;
-	GSList *iter;
+	gchar *device;
 
-	priv = BRASERO_MEDIUM_MONITOR_PRIVATE (self);
+	BRASERO_MEDIA_LOG ("Drive removal signal");
 
-	BRASERO_MEDIA_LOG ("Device removed");
-
-	for (iter = priv->drives; iter; iter = next) {
-		GDrive *gdrive_iter;
-		BraseroDrive *drive;
-
-		drive = iter->data;
-		next = iter->next;
-
-		gdrive_iter = brasero_drive_get_gdrive (drive);
-		if (gdrive == gdrive_iter) {
-			BraseroMedium *medium;
-
-			BRASERO_MEDIA_LOG ("Drive removed");
-			medium = brasero_drive_get_medium (drive);
-			if (medium)
-				g_signal_emit (self,
-					       medium_monitor_signals [MEDIUM_REMOVED],
-					       0,
-					       medium);
-
-			priv->drives = g_slist_remove (priv->drives, drive);
-			g_signal_emit (self,
-				       medium_monitor_signals [DRIVE_REMOVED],
-				       0,
-				       drive);
-			g_object_unref (drive);
-		}
-	}
+	device = g_drive_get_identifier (gdrive, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+	brasero_medium_monitor_device_removed (self, device, gdrive);
+	g_free (device);
 }
 
 static void
@@ -432,6 +576,7 @@ brasero_medium_monitor_init (BraseroMediumMonitor *object)
 {
 	GList *iter;
 	GList *drives;
+	GList *volumes;
 	BraseroDrive *drive;
 	BraseroMediumMonitorPrivate *priv;
 
@@ -439,9 +584,6 @@ brasero_medium_monitor_init (BraseroMediumMonitor *object)
 
 	BRASERO_MEDIA_LOG ("Probing drives and media");
 
-	/* This must done early on. GVolumeMonitor when it relies on HAL (like
-	 * us) must be able to update its list of volumes before us so it must
-	 * connect to HAL before us. */
 	priv->gmonitor = g_volume_monitor_get ();
 
 	drives = g_volume_monitor_get_connected_drives (priv->gmonitor);
@@ -449,12 +591,56 @@ brasero_medium_monitor_init (BraseroMediumMonitor *object)
 
 	for (iter = drives; iter; iter = iter->next) {
 		GDrive *gdrive;
+		gchar *device;
 
 		gdrive = iter->data;
-		if (brasero_medium_monitor_is_drive (object, gdrive)) {
+
+		device = g_drive_get_identifier (gdrive, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+		if (brasero_medium_monitor_is_drive (object, device)) {
 			drive = g_object_new (BRASERO_TYPE_DRIVE,
+			                      "device", device,
 					      "gdrive", gdrive,
 					      NULL);
+
+			priv->drives = g_slist_prepend (priv->drives, drive);
+
+			g_signal_connect (drive,
+					  "medium-added",
+					  G_CALLBACK (brasero_medium_monitor_medium_added_cb),
+					  object);
+			g_signal_connect (drive,
+					  "medium-removed",
+					  G_CALLBACK (brasero_medium_monitor_medium_removed_cb),
+					  object);
+		}
+		g_free (device);
+	}
+	g_list_foreach (drives, (GFunc) g_object_unref, NULL);
+	g_list_free (drives);
+
+	volumes = g_volume_monitor_get_volumes (priv->gmonitor);
+	BRASERO_MEDIA_LOG ("Found %d volumes", g_list_length (volumes));
+
+	for (iter = volumes; iter; iter = iter->next) {
+		GVolume *gvolume;
+		gchar *device;
+
+		gvolume = iter->data;
+		device = g_volume_get_identifier (gvolume, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE);
+
+		/* make sure it isn't already in our list */
+		drive = brasero_medium_monitor_get_drive (object, device);
+		if (drive) {
+			g_free (device);
+			g_object_unref (drive);
+			continue;
+		}
+
+		if (brasero_medium_monitor_is_drive (object, device)) {
+			drive = g_object_new (BRASERO_TYPE_DRIVE,
+			                      "device", device,
+			                      "gdrive", NULL,
+					      NULL);
 			priv->drives = g_slist_prepend (priv->drives, drive);
 
 			g_signal_connect (drive,
@@ -466,10 +652,20 @@ brasero_medium_monitor_init (BraseroMediumMonitor *object)
 					  G_CALLBACK (brasero_medium_monitor_medium_removed_cb),
 					  object);
 		}
-	}
-	g_list_foreach (drives, (GFunc) g_object_unref, NULL);
-	g_list_free (drives);
 
+		g_free (device);
+	}
+	g_list_foreach (volumes, (GFunc) g_object_unref, NULL);
+	g_list_free (volumes);
+
+	g_signal_connect (priv->gmonitor,
+			  "volume-added",
+			  G_CALLBACK (brasero_medium_monitor_volume_added_cb),
+			  object);
+	g_signal_connect (priv->gmonitor,
+			  "volume-removed",
+			  G_CALLBACK (brasero_medium_monitor_volume_removed_cb),
+			  object);
 	g_signal_connect (priv->gmonitor,
 			  "drive-connected",
 			  G_CALLBACK (brasero_medium_monitor_connected_cb),
@@ -480,7 +676,9 @@ brasero_medium_monitor_init (BraseroMediumMonitor *object)
 			  object);
 
 	/* add fake/file drive */
-	drive = g_object_new (BRASERO_TYPE_DRIVE, NULL);
+	drive = g_object_new (BRASERO_TYPE_DRIVE,
+	                      "device", NULL,
+	                      NULL);
 	priv->drives = g_slist_prepend (priv->drives, drive);
 
 	return;
@@ -493,6 +691,16 @@ brasero_medium_monitor_finalize (GObject *object)
 
 	priv = BRASERO_MEDIUM_MONITOR_PRIVATE (object);
 
+	if (priv->waiting_removal_id) {
+		g_source_remove (priv->waiting_removal_id);
+		priv->waiting_removal_id = 0;
+	}
+
+	if (priv->waiting_removal) {
+		g_slist_free (priv->waiting_removal);
+		priv->waiting_removal = NULL;
+	}
+
 	if (priv->drives) {
 		g_slist_foreach (priv->drives, (GFunc) g_object_unref, NULL);
 		g_slist_free (priv->drives);
@@ -500,6 +708,12 @@ brasero_medium_monitor_finalize (GObject *object)
 	}
 
 	if (priv->gmonitor) {
+		g_signal_handlers_disconnect_by_func (priv->gmonitor,
+		                                      brasero_medium_monitor_volume_added_cb,
+		                                      object);
+		g_signal_handlers_disconnect_by_func (priv->gmonitor,
+		                                      brasero_medium_monitor_volume_removed_cb,
+		                                      object);
 		g_signal_handlers_disconnect_by_func (priv->gmonitor,
 		                                      brasero_medium_monitor_connected_cb,
 		                                      object);
