@@ -1,24 +1,28 @@
-/***************************************************************************
- *            wodim.c
- *
- *  dim jan 22 15:22:52 2006
- *  Copyright  2006  Rouquier Philippe
- *  brasero-app@wanadoo.fr
- ***************************************************************************/
-
+/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 8; tab-width: 8 -*- */
 /*
- *  Brasero is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License as published by
- *  the Free Software Foundation; either version 2 of the License, or
- *  (at your option) any later version.
+ * Libbrasero-burn
+ * Copyright (C) Philippe Rouquier 2005-2009 <bonfire-app@wanadoo.fr>
  *
- *  Brasero is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU Library General Public License for more details.
+ * Libbrasero-burn is free software; you can redistribute it and/or modify
+ * it under the terms of the GNU General Public License as published by
+ * the Free Software Foundation; either version 2 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to:
+ * The Libbrasero-burn authors hereby grant permission for non-GPL compatible
+ * GStreamer plugins to be used and distributed together with GStreamer
+ * and Libbrasero-burn. This permission is above and beyond the permissions granted
+ * by the GPL license by which Libbrasero-burn is covered. If you modify this code
+ * you may extend this exception to your version of the code, but you are not
+ * obligated to do so. If you do not wish to do so, delete this exception
+ * statement from your version.
+ * 
+ * Libbrasero-burn is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU Library General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to:
  * 	The Free Software Foundation, Inc.,
  * 	51 Franklin Street, Fifth Floor
  * 	Boston, MA  02110-1301, USA.
@@ -60,8 +64,8 @@
 BRASERO_PLUGIN_BOILERPLATE (BraseroWodim, brasero_wodim, BRASERO_TYPE_PROCESS, BraseroProcess);
 
 struct _BraseroWodimPrivate {
-	gint64 current_track_end_pos;
-	gint64 current_track_written;
+	goffset current_track_end_pos;
+	goffset current_track_written;
 
 	gint current_track_num;
 	gint track_count;
@@ -207,38 +211,44 @@ brasero_wodim_stderr_read (BraseroProcess *process, const gchar *line)
 
 static void
 brasero_wodim_compute (BraseroWodim *wodim,
-		       gint mb_written,
-		       gint mb_total,
-		       gint track_num)
+		       goffset mb_written,
+		       goffset mb_total,
+		       goffset track_num)
 {
 	gboolean track_num_changed = FALSE;
 	BraseroWodimPrivate *priv;
 	BraseroJobAction action;
-	gint64 this_remain;
-	gint64 bytes;
-	gint64 total;
+	goffset this_remain;
+	goffset bytes;
+	goffset total;
 
 	priv = BRASERO_WODIM_PRIVATE (wodim);
 	if (mb_total <= 0)
 		return;
 
-	total = mb_total * 1048576;
+	total = mb_total * (goffset) 1048576LL;
 
 	if (track_num > priv->current_track_num) {
 		track_num_changed = TRUE;
 		priv->current_track_num = track_num;
-		priv->current_track_end_pos += mb_total * 1048576;
+		priv->current_track_end_pos += mb_total * (goffset) 1048576LL;
 	}
 
-	this_remain = (mb_total - mb_written) * 1048576;
+	this_remain = (mb_total - mb_written) * (goffset) 1048576LL;
 	bytes = (total - priv->current_track_end_pos) + this_remain;
 	brasero_job_set_written_session (BRASERO_JOB (wodim), total - bytes);
 
 	brasero_job_get_action (BRASERO_JOB (wodim), &action);
 	if (action == BRASERO_JOB_ACTION_RECORD) {
 		gchar *action_string;
+		gchar *track_num_str;
+
+		track_num_str = g_strdup_printf ("%02" G_GOFFSET_FORMAT, track_num);
+
+		/* Translators: %s is the number of the track */
+		action_string = g_strdup_printf (_("Writing track %s"), track_num_str);
+		g_free (track_num_str);
 
-		action_string = g_strdup_printf ("Writing track %02i", track_num);
 		brasero_job_set_current_action (BRASERO_JOB (wodim),
 						BRASERO_BURN_ACTION_RECORDING,
 						action_string,
@@ -275,7 +285,7 @@ brasero_wodim_stdout_read (BraseroProcess *process, const gchar *line)
 			       (gdouble) CD_RATE;
 		brasero_job_set_rate (BRASERO_JOB (wodim), current_rate);
 
-		priv->current_track_written = mb_written * 1048576;
+		priv->current_track_written = (goffset) mb_written * (goffset) 1048576LL;
 		brasero_wodim_compute (wodim,
 				       mb_written,
 				       mb_total,
@@ -293,7 +303,7 @@ brasero_wodim_stdout_read (BraseroProcess *process, const gchar *line)
 			       (gdouble) CD_RATE;
 		brasero_job_set_rate (BRASERO_JOB (wodim), current_rate);
 
-		priv->current_track_written = mb_written * 1048576;
+		priv->current_track_written = (goffset) mb_written * (goffset) 1048576LL;
 		if (brasero_job_get_fd_in (BRASERO_JOB (wodim), NULL) == BRASERO_BURN_OK) {
 			goffset bytes = 0;
 
@@ -301,7 +311,7 @@ brasero_wodim_stdout_read (BraseroProcess *process, const gchar *line)
 			brasero_job_get_session_output_size (BRASERO_JOB (wodim),
 							     NULL,
 							     &bytes);
-			mb_total = bytes / 1048576;
+			mb_total = bytes / (goffset) 1048576LL;
 			brasero_wodim_compute (wodim,
 					       mb_written,
 					       mb_total,
@@ -359,10 +369,15 @@ brasero_wodim_stdout_read (BraseroProcess *process, const gchar *line)
 	}
 	else if (g_str_has_prefix (line, "Fixating...")
 	     ||  g_str_has_prefix (line, "Writing Leadout...")) {
-		brasero_job_set_current_action (BRASERO_JOB (process),
-						BRASERO_BURN_ACTION_FIXATING,
-						NULL,
-						FALSE);
+		BraseroJobAction action;
+
+		/* Do this to avoid strange things to appear when erasing */
+		brasero_job_get_action (BRASERO_JOB (wodim), &action);
+		if (action == BRASERO_JOB_ACTION_RECORD)
+			brasero_job_set_current_action (BRASERO_JOB (process),
+							BRASERO_BURN_ACTION_FIXATING,
+							NULL,
+							FALSE);
 	}
 	else if (g_str_has_prefix (line, "Last chance to quit, ")) {
 		brasero_job_set_dangerous (BRASERO_JOB (process), TRUE);
