@@ -40,7 +40,12 @@
 #include <gtk/gtk.h>
 
 #include "brasero-misc.h"
-#include "burn-basics.h"
+#include "brasero-io.h"
+
+#include "brasero-tags.h"
+#include "brasero-track-stream-cfg.h"
+#include "brasero-session-cfg.h"
+
 #include "brasero-app.h"
 #include "brasero-disc.h"
 #include "brasero-audio-disc.h"
@@ -48,12 +53,8 @@
 #include "brasero-utils.h"
 #include "brasero-multi-song-props.h"
 #include "brasero-song-properties.h"
-#include "brasero-io.h"
 #include "brasero-split-dialog.h"
-#include "brasero-track-stream-cfg.h"
-#include "brasero-session-cfg.h"
 #include "brasero-video-tree-model.h"
-#include "brasero-tags.h"
 
 #include "eggtreemultidnd.h"
 
@@ -430,7 +431,11 @@ brasero_audio_disc_init (BraseroAudioDisc *obj)
 
 	gtk_tree_view_column_pack_end (column, renderer, TRUE);
 	gtk_tree_view_column_add_attribute (column, renderer,
-					    "markup", BRASERO_VIDEO_TREE_MODEL_NAME);
+					    "text", BRASERO_VIDEO_TREE_MODEL_NAME);
+	gtk_tree_view_column_add_attribute (column, renderer,
+					    "style", BRASERO_VIDEO_TREE_MODEL_STYLE);
+	gtk_tree_view_column_add_attribute (column, renderer,
+					    "weight", BRASERO_VIDEO_TREE_MODEL_WEIGHT);
 	gtk_tree_view_column_add_attribute (column, renderer,
 					    "editable", BRASERO_VIDEO_TREE_MODEL_EDITABLE);
 	gtk_tree_view_column_set_title (column, _("Title"));
@@ -486,10 +491,6 @@ brasero_audio_disc_init (BraseroAudioDisc *obj)
 	gtk_container_add (GTK_CONTAINER (scroll), obj->priv->tree);
 
 	gtk_box_pack_start (GTK_BOX (obj), scroll, TRUE, TRUE, 0);
-	/*gtk_notebook_append_page (GTK_NOTEBOOK (obj->priv->notebook),
-				  scroll,
-				  NULL);
-	gtk_notebook_set_current_page (GTK_NOTEBOOK (obj->priv->notebook), 0);*/
 
 	/* dnd */
 	gtk_tree_view_enable_model_drag_dest (GTK_TREE_VIEW (obj->priv->tree),
@@ -827,6 +828,12 @@ brasero_audio_disc_session_changed (BraseroSessionCfg *session,
 			gchar *uri;
 
 			uri = brasero_track_stream_get_source (track, TRUE);
+
+			/* Remove the track now otherwise on each session change we'll get the
+			 * same message over and over again. */
+			brasero_burn_session_remove_track (BRASERO_BURN_SESSION (session),
+							   BRASERO_TRACK (track));
+
 			error = brasero_status_get_error (status);
 			if (!error)
 				brasero_audio_disc_file_type_error_dialog (self, uri);
@@ -852,8 +859,6 @@ brasero_audio_disc_session_changed (BraseroSessionCfg *session,
 								      uri,
 								      error);
 
-			brasero_burn_session_remove_track (BRASERO_BURN_SESSION (session),
-							   BRASERO_TRACK (track));
 			g_error_free (error);
 			g_free (uri);
 			continue;
